function [Postdraw,Pr] = GibbsSamplerMSskew(M,Gibbs)

% Preallocation
% --
location_store  = NaN(M.nreg,Gibbs.n);
invscale2_store = NaN(M.nreg,Gibbs.n);
scale_store     = NaN(M.nreg,Gibbs.n);
shape_store     = NaN(M.nreg,Gibbs.n);
phi_store       = NaN(1,Gibbs.n);
a1_store        = NaN(3,Gibbs.n);
a2_store        = NaN(3,Gibbs.n);
b1_store        = NaN(2,3,Gibbs.n);
b2_store        = NaN(2,3,Gibbs.n);
s_store         = NaN(M.t,Gibbs.n);

for i=1:Gibbs.n
    
    if mod(i,1000)==0
        fprintf(['%d out of %d draws [',datestr(M.DatRT),']\n'],i,Gibbs.n);
    end
    
    % step 1. Generate historical path of unobservable state, s
    M = gbs_s_single_move(M);
    
    % step 2. Generate historical path of unobservable state, z
    M = gbs_z(M);
    
    % step 3. Generate parameters governing transition matrix, Q
    M = gbs_transition_matrix(M);

    % step 4. Generate location parameters, xi
    M = gbs_location(M);

    % step 5. Generate scale parameters, omega
    M = gbs_invscale2(M);
    
    % step 6. Generate shape parameters, alpha
    if(M.name_file~="normal")
        M = gbs_shape(M);
    end

    % store Gibbs draws
    location_store(:,i)  = M.params.location;
    invscale2_store(:,i) = M.params.invscale2;
    scale_store(:,i)     = M.params.scale;
    shape_store(:,i)     = M.params.shape;
    phi_store(:,i)       = M.params.phi;
    a1_store(:,i)        = M.params.a1;
    a2_store(:,i)        = M.params.a2;
    b1_store(:,:,i)      = M.params.b1;
    b2_store(:,:,i)      = M.params.b2;
    s_store(:,i)         = M.s;
    
end

fss = (Gibbs.n-Gibbs.nburn)/Gibbs.nuse;

% Keep only every nuse-th draw
%  --
a1gbs        = forevery(a1_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
a2gbs        = forevery(a2_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
b11gbs       = forevery(squeeze(b1_store(1,:,Gibbs.nburn+1:Gibbs.n))',Gibbs.nuse);
b12gbs       = forevery(squeeze(b1_store(2,:,Gibbs.nburn+1:Gibbs.n))',Gibbs.nuse);
b21gbs       = forevery(squeeze(b2_store(1,:,Gibbs.nburn+1:Gibbs.n))',Gibbs.nuse);
b22gbs       = forevery(squeeze(b2_store(2,:,Gibbs.nburn+1:Gibbs.n))',Gibbs.nuse);
phigbs       = forevery(phi_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
invscale2gbs = forevery(invscale2_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
scalegbs     = forevery(scale_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
shapegbs     = forevery(shape_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
locationgbs  = forevery(location_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
sgbs         = forevery(s_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);

% Recover regimes for each MS process
% --
s_location = NaN(M.t,fss);
s_scale    = NaN(M.t,fss);
s_shape    = NaN(M.t,fss);
for id=1:fss
    s_location(:,id) = sum(sgbs(id,:)'==find(M.regimes(2,:)),2) + 2*(1-sum(sgbs(id,:)'==find(M.regimes(2,:)),2));
    s_scale(:,id)    = sum(sgbs(id,:)'==find(M.regimes(3,:)),2) + 2*(1-sum(sgbs(id,:)'==find(M.regimes(3,:)),2));
    s_shape(:,id)    = sum(sgbs(id,:)'==find(M.regimes(1,:)),2) + 2*(1-sum(sgbs(id,:)'==find(M.regimes(1,:)),2));
end

% Compute probabilities for being in each regime
% --
Pr_location = NaN(M.t,2);
Pr_scale    = NaN(M.t,2);
Pr_shape    = NaN(M.t,2);
for it=1:M.t
    Pr_location(it,:) = [sum(s_location(it,:)==1)/fss;1-sum(s_location(it,:)==1)/fss];
    Pr_scale(it,:)    = [sum(s_scale(it,:)==1)/fss;1-sum(s_scale(it,:)==1)/fss];
    Pr_shape(it,:)    = [sum(s_shape(it,:)==1)/fss;1-sum(s_shape(it,:)==1)/fss];
end

Pr = cell(3,1);
Pr{1} = Pr_location; Pr{2} = Pr_scale; Pr{3} = Pr_shape; 

% Display Table 
Postdraw = [locationgbs(:,[1,3]),invscale2gbs(:,[1,2]),shapegbs(:,[1,5]),phigbs, ...
            a1gbs(:,1),a2gbs(:,1),b11gbs(:,1),b12gbs(:,1),b21gbs(:,1),b22gbs(:,1), ...
            a1gbs(:,2),a2gbs(:,2),b11gbs(:,2),b12gbs(:,2),b21gbs(:,2),b22gbs(:,2), ...
            a1gbs(:,3),a2gbs(:,3),b11gbs(:,3),b12gbs(:,3),b21gbs(:,3),b22gbs(:,3)];

% Save Results
% --
filename=['results/models/',M.name_file,'/real-time/gibbs/',datestr(M.DatRT),'.mat'];
save(filename,'a1gbs','a2gbs','b11gbs','b12gbs','b21gbs','b22gbs','phigbs', ...
    'invscale2gbs','scalegbs','shapegbs','locationgbs','sgbs',...
    'Pr_location','Pr_scale','Pr_shape','s_location', ...
    's_scale','s_shape','fss');                               

Postdraw_med = median(Postdraw,1);
Postdraw_mea = mean(Postdraw,1);
Postdraw_lim = hpdint(Postdraw,0.68,0);

% Save estimated parameters
% --
cnames={'Mean','Median','Lower','Upper'};
pnames={'location(1)','location(2)', ...
    'invscale2(1)','invscale2(2)', ...
    'shape(1)','shape(2)', ...
    'phi1', ...
    'a1(shape)','a2(shape)', ...
    'b1(shape,gdp)','b1(shape,ciss)', ...
    'b2(shape,gdp)','b2(shape,ciss)', ...
    'a1(location)','a2(location)', ...
    'b1(location,gdp)','b1(location,ciss)', ...
    'b2(location,gdp)','b2(location,ciss)', ...
    'a1(scale)','a2(scale)', ...
    'b1(scale,gdp)','b1(scale,ciss)', ...
    'b2(scale,gdp)','b2(scale,ciss)'};
out = [Postdraw_mea',Postdraw_med',Postdraw_lim'];
T = table(pnames',out(:,1),out(:,2),out(:,3),out(:,4),'VariableNames',['Parameters',cnames]);

writetable(T,['results/models/',M.name_file,'/real-time/gibbs/',datestr(M.DatRT),'.xlsx']);

% Display Gibbs Results
% --
idx_fig  = 1;
path_fig = ['results/models/',M.name_file,'/real-time/gibbs/figures'];

fig_gibbs;