%
% Code to replicate Lhuissier (2022): "Financial Conditions and 
% Macroeconomic Downside Risks in the Euro Area", European Economic Review
%   
% --> Gibbs-Sampling Procedure
%
%
% Stephane Lhuissier, August 2021

% -------------------------------------------------------------------------
% Housekeeping
% -------------------------------------------------------------------------

clear variables; clc;

addpath 'library/'

% -------------------------------------------------------------------------
% Preliminary work
% -------------------------------------------------------------------------

% Model Specification
M.name_file = 'benchmark';

% Options for Gibbs sampler
% --
rng(123);

Gibbs.n            = 55000;
Gibbs.nburn        = 5000;
Gibbs.nuse         = 10;

% Load Results (obtained from run_optimization.m)
% --
load(['results/models/',M.name_file,'/posterior_mode.mat'])

% Load Data
% --
M.yt               = options_.data;
M.t                = size(M.yt,1);

% Assign Values to Parameters
% --
param              = Postmode;
M.Postmode         = Postmode;
M.Dat              = Dat;
yt                 = M.yt;

M.params.location  = [param(1); param(1); param(2); param(2); param(1); param(1); param(2); param(2)]; 
M.params.invscale2 = [param(3); param(4); param(3); param(4); param(3); param(4); param(3); param(4)];
M.params.scale     = sqrt(1./M.params.invscale2);
M.params.shape     = [param(5); param(5); param(5); param(5); param(6); param(6); param(6); param(6)];
M.params.adjshape  = M.params.shape./(sqrt(1+M.params.shape.^2));

M.regimes = [1,1,1,1,0,0,0,0
             1,1,0,0,1,1,0,0
             1,0,1,0,1,0,1,0];
         
M.nprocess = 3;

M.params.phi       = param(7);

M.params.a1        = [param(8); param(14);param(20)];
M.params.a2        = [param(9); param(15);param(21)];
M.params.b1        = [param(10) param(16) param(22)   % b11
                      param(11) param(17) param(23)]; % b12
M.params.b2        = [param(12) param(18) param(24)   % b21
                      param(13) param(19) param(25)]; % b22

Q                  = tran_p2q(M.params.a1,M.params.a2,M.params.b1,M.params.b2,zeros(1,2));
M.s                = mcdraw_f2_most_likely(Q,pr_st0);
M.fp               = pr_st0;
M.nreg             = size(pr_st0,2);

% Define Prior (Same as the one used in the optimization routine)
% --
M.prior.location.mu  = options_.pmean(1:2);
M.prior.location.sig = options_.pstdd(1:2).^2;
M.prior.invscale2.a  = options_.pmean(3:4);
M.prior.invscale2.b  = options_.pstdd(3:4);
M.prior.shape.mu     = options_.pmean(5:6);
M.prior.shape.sig    = options_.pstdd(5:6);
M.prior.phi.mu       = options_.pmean(7);
M.prior.phi.sig      = options_.pstdd(7).^2;
M.prior.a1.mu        = options_.pmean([8,14,20]);
M.prior.a1.sig       = options_.pstdd([8,14,20]);
M.prior.a2.mu        = options_.pmean([9,15,21]);
M.prior.a2.sig       = options_.pstdd([9,15,21]);
M.prior.b1.mu        = [options_.pmean([10,16,22]),options_.pmean([11,17,23])];
M.prior.b1.sig       = [options_.pstdd([10,16,22]),options_.pstdd([11,17,23])];
M.prior.b2.mu        = [options_.pmean([12,18,24]),options_.pmean([13,19,25])];
M.prior.b2.sig       = [options_.pstdd([12,18,24]),options_.pstdd([13,19,25])];

M.prior.transition_matrix.mu  = [M.prior.a1.mu,M.prior.a2.mu,M.prior.b1.mu(:,1),M.prior.b1.mu(:,2),M.prior.b2.mu(:,1),M.prior.b2.mu(:,2)]';
M.prior.transition_matrix.sig = NaN(6,6,3);
for i=1:size(M.regimes,1)
    M.prior.transition_matrix.sig(:,:,i) = diag([M.prior.a1.sig(i),M.prior.a2.sig(i),M.prior.b1.sig(i,1),M.prior.b1.sig(i,2),M.prior.b2.sig(i,1),M.prior.b2.sig(i,2)]).^2;
end

% -------------------------------------------------------------------------
% Gibbs Sampling Procedure
% -------------------------------------------------------------------------

% Preallocation
% --
location_store  = NaN(M.nreg,Gibbs.n);
invscale2_store = NaN(M.nreg,Gibbs.n);
scale_store     = NaN(M.nreg,Gibbs.n);
shape_store     = NaN(M.nreg,Gibbs.n);
phi_store       = NaN(1,Gibbs.n);
a1_store        = NaN(3,Gibbs.n);
a2_store        = NaN(3,Gibbs.n);
b1_store        = NaN(2,3,Gibbs.n);
b2_store        = NaN(2,3,Gibbs.n);
rej_store       = NaN(2,Gibbs.n);
s_store         = NaN(M.t,Gibbs.n);

% Running
% --
tic
fprintf('Gibbs sampler starts here...\n');
for i=1:Gibbs.n
    
    if mod(i,1000)==0
        fprintf('%d out of %d draws.\n',i,Gibbs.n);
    end
    
    % step 1. Generate historical path of unobservable state, s
    M = gbs_s_single_move(M);
    
    % step 2. Generate historical path of unobservable state, z
    M = gbs_z(M);
    
    % step 3. Generate parameters governing transition matrix, Q
    M = gbs_transition_matrix(M);

    % step 4. Generate location parameters, xi
    M = gbs_location(M);

    % step 5. Generate scale parameters, omega
    M = gbs_invscale2(M);
    
    % step 6. Generate shape parameters, alpha
    if(M.name_file~="normal")
        M = gbs_shape(M);
    end
    
    % store Gibbs draws
    location_store(:,i)  = M.params.location;
    invscale2_store(:,i) = M.params.invscale2;
    scale_store(:,i)     = M.params.scale;
    shape_store(:,i)     = M.params.shape;
    phi_store(:,i)       = M.params.phi;
    a1_store(:,i)        = M.params.a1;
    a2_store(:,i)        = M.params.a2;
    b1_store(:,:,i)      = M.params.b1;
    b2_store(:,:,i)      = M.params.b2;
    s_store(:,i)         = M.s;
    
end
fprintf('\nElapsed time: %s.\n',secs2hms(toc))

fss = (Gibbs.n-Gibbs.nburn)/Gibbs.nuse;

% Keep only every nuse-th draw
%  --
a1gbs        = forevery(a1_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
a2gbs        = forevery(a2_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
b11gbs       = forevery(squeeze(b1_store(1,:,Gibbs.nburn+1:Gibbs.n))',Gibbs.nuse);
b12gbs       = forevery(squeeze(b1_store(2,:,Gibbs.nburn+1:Gibbs.n))',Gibbs.nuse);
b21gbs       = forevery(squeeze(b2_store(1,:,Gibbs.nburn+1:Gibbs.n))',Gibbs.nuse);
b22gbs       = forevery(squeeze(b2_store(2,:,Gibbs.nburn+1:Gibbs.n))',Gibbs.nuse);
phigbs       = forevery(phi_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
invscale2gbs = forevery(invscale2_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
scalegbs     = forevery(scale_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
shapegbs     = forevery(shape_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
locationgbs  = forevery(location_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);
sgbs         = forevery(s_store(:,Gibbs.nburn+1:Gibbs.n)',Gibbs.nuse);

% Recover regimes for each MS process
% --
s_location = NaN(M.t,fss);
s_scale    = NaN(M.t,fss);
s_shape    = NaN(M.t,fss);
for id=1:fss
    s_location(:,id) = sum(sgbs(id,:)'==find(M.regimes(2,:)),2) + 2*(1-sum(sgbs(id,:)'==find(M.regimes(2,:)),2));
    s_scale(:,id)    = sum(sgbs(id,:)'==find(M.regimes(3,:)),2) + 2*(1-sum(sgbs(id,:)'==find(M.regimes(3,:)),2));
    s_shape(:,id)    = sum(sgbs(id,:)'==find(M.regimes(1,:)),2) + 2*(1-sum(sgbs(id,:)'==find(M.regimes(1,:)),2));
end

% Compute probabilities for being in each regime
% --
Pr_location = NaN(M.t,2);
Pr_scale    = NaN(M.t,2);
Pr_shape    = NaN(M.t,2);
for it=1:M.t
    Pr_location(it,:) = [sum(s_location(it,:)==1)/fss;1-sum(s_location(it,:)==1)/fss];
    Pr_scale(it,:)    = [sum(s_scale(it,:)==1)/fss;1-sum(s_scale(it,:)==1)/fss];
    Pr_shape(it,:)    = [sum(s_shape(it,:)==1)/fss;1-sum(s_shape(it,:)==1)/fss];
end

% -------------------------------------------------------------------------
% Results
% -------------------------------------------------------------------------

% Display Gibbs Results
% --
idx_fig  = 0;
fig_gibbs;

% Save Results
% --
save(['results/models/',M.name_file,'/gibbs_draws.mat'],'a1gbs','a2gbs','b11gbs','b12gbs','b21gbs','b22gbs','phigbs', ...
                                   'invscale2gbs','scalegbs','shapegbs','locationgbs','sgbs',...
                                   'Pr_location','Pr_scale','Pr_shape','s_location', ...
                                   's_scale','s_shape','fss','Dat','yt');

% Display Table 
Postdraw = [locationgbs(:,[1,3]),invscale2gbs(:,[1,2]),shapegbs(:,[1,5]),phigbs, ...
            a1gbs(:,1),a2gbs(:,1),b11gbs(:,1),b12gbs(:,1),b21gbs(:,1),b22gbs(:,1), ...
            a1gbs(:,2),a2gbs(:,2),b11gbs(:,2),b12gbs(:,2),b21gbs(:,2),b22gbs(:,2), ...
            a1gbs(:,3),a2gbs(:,3),b11gbs(:,3),b12gbs(:,3),b21gbs(:,3),b22gbs(:,3)];

Postdraw_med = median(Postdraw,1);
Postdraw_mea = mean(Postdraw,1);
Postdraw_lim = hpdint(Postdraw,0.68,0);

fprintf('\nEmpirical moments...\n')
cnames={'Mean','Median','Lower','Upper'};
pnames={'location(1)','location(2)','invscale2(1)','invscale2(2)','shape(1)','shape(2)',...
        'phi', ...
        'a1(shape)','a2(shape)','b1(shape,gdp)', 'b1(shape,ciss)', 'b2(shape,gdp)','b2(shape,ciss)', ...
        'a1(location)','a2(location)','b1(location,gdp)', 'b1(location,ciss)', 'b2(location,gdp)','b2(location,ciss)', ...
        'a1(scale)','a2(scale)','b1(scale,gdp)', 'b1(scale,ciss)', 'b2(scale,gdp)','b2(scale,ciss)'};
out = [Postdraw_mea',Postdraw_med',Postdraw_lim'];
r   = printresults3(out,cnames,pnames); disp(r);
T = table(pnames',out(:,1),out(:,2),out(:,3),out(:,4),'VariableNames',['Parameters',cnames]);
writetable(T,['results/models/',M.name_file,'/Gibbs_moments68.xlsx']);

% WAIC 
% --
lnlik = zeros(M.t,fss);
for i=1:fss
    [~,~,~,~,lnlik(:,i)] = ms3p_tvtp_lik(Postdraw(i,:),M.yt);
end

elpd_waici   = log(mean(exp(lnlik),2))-var(lnlik,0,2);
waic         = sum(elpd_waici);
se_elpd_waic = sqrt(M.t*var(elpd_waici));

fprintf('WAIC is %f.\n',waic);
fprintf('standard errors of WAIC is %f.\n',se_elpd_waic);

save(['results/models/',M.name_file,'/WAIC.mat'],'waic','se_elpd_waic','elpd_waici');
