%
% Code to replicate Lhuissier (2022): "Financial Conditions and
% Macroeconomic Downside Risks in the Euro Area", European Economic Review
%
% --> Procedure for computing moments, forecast metrics and other
% statistics
%
%
% Stephane Lhuissier, August 2021
%
% Corrigendum, Stephane Lhuissier, August 2022

% -------------------------------------------------------------------------
% Housekeeping
% -------------------------------------------------------------------------

clear variables; clc;

addpath 'library/'
addpath 'library/skn-library/'

% -------------------------------------------------------------------------
% Settings
% -------------------------------------------------------------------------

% Model Specification
name_file = 'benchmark';

% Saved Gibbs results
results_root = 'results';       % Root folder with MCMC results
save_root = 'results';    % Root folder to save computations in

% Target probability for expected shortfall/longrise
alpha = 0.05;

% Interval width for approximation to expected shortfall/longrise integral
delta1 = 0.01;

% probability bands
p = [5 16 50 84 95];

deltaYY = 0.1;         % Step size and grid to use for computing CDFs/PDFs
YY = (-20):deltaYY:20;

% Number of regimes
nregimes = 8;

% Number of workers
nworkers = 16;

% -------------------------------------------------------------------------
% Load MCMC results
% -------------------------------------------------------------------------

load([results_root,'/models/',name_file,'/gibbs_draws.mat'], ...
    'a1gbs','a2gbs','b11gbs','b12gbs','b21gbs','b22gbs',...
    'locationgbs','phigbs','scalegbs','shapegbs','sgbs','yt','Dat');

T = size(yt,1);
ytRealized=yt(:,1);
numdraw = size(locationgbs,1);

% -------------------------------------------------------------------------
% h=1: Numerical computations
% -------------------------------------------------------------------------

samples_score = nan(numdraw,T);

samples_EsfST = nan(numdraw,T);
samples_EljST = nan(numdraw,T);
samples_PrRec = nan(numdraw,T);

samples_mean          = nan(numdraw,T);
samples_variance      = nan(numdraw,T);
samples_skewness      = nan(numdraw,T);
samples_skewness_coef = nan(numdraw,T);
samples_kurtosis      = nan(numdraw,T);
samples_kurtosis_coef = nan(numdraw,T);

samples_pst           = nan(T,length(YY),numdraw);
samples_quint         = nan(T,length(p),numdraw);

samples_IC            = nan(numdraw,T);
samples_TLF           = nan(numdraw,T);
samples_GZF           = nan(numdraw,T);
samples_ALS           = nan(numdraw,T);

tic
disp('h = 1: Computing probabilities, quantiles, shortfall, negative growth numerically');

parfor(ip=1:numdraw,nworkers) % Loop over posterior draws
    
    % load parameters
    samples_location_coef = locationgbs(ip,:);
    samples_scale_coef    = scalegbs(ip,:);
    samples_shape_coef    = shapegbs(ip,:);
    samples_s_coef        = sgbs(ip,:);
    samples_a1_coef       = a1gbs(ip,:);
    samples_a2_coef       = a2gbs(ip,:);
    samples_b11_coef      = b11gbs(ip,:);
    samples_b12_coef      = b12gbs(ip,:);
    samples_b21_coef      = b21gbs(ip,:);
    samples_b22_coef      = b22gbs(ip,:);
    
    % Recover real-time probabilities
    [~, pr_st0] = ms3p_tvtp_lik_alt(samples_location_coef, samples_scale_coef, samples_shape_coef, 0, samples_a1_coef, samples_a2_coef, samples_b11_coef, samples_b12_coef, samples_b21_coef, samples_b22_coef, yt);
    
    % Compute first four moments—mean, variance, skewness, and kurtosis—
    [samples_mean(ip,:), samples_variance(ip,:), samples_skewness(ip,:), samples_skewness_coef(ip,:), samples_kurtosis(ip,:), samples_kurtosis_coef(ip,:)] = ms_skn_moments(pr_st0, samples_location_coef, samples_scale_coef, samples_shape_coef);
    
    % Compute predictive score
    samples_score(ip,:) = sum(pr_st0.*dsn(ytRealized,samples_location_coef,samples_scale_coef,samples_shape_coef),2);
    
    % Predictive densities conditional on each regime
    samples_pdcond = nan(nregimes,length(YY));
    
    for jp=1:nregimes
        % Predictive densities conditional on each regime
        samples_pdcond(jp,:) = dsn(YY,samples_location_coef(jp),samples_scale_coef(jp),samples_shape_coef(jp));
    end
    
    for k=1:T
        
        % compute densities
        samples_pst(k,:,ip) = pr_st0(k,:)*samples_pdcond;
        
        % generate a mixture of skew-normal distributions
        samples_data = gen_mixture(samples_location_coef,samples_scale_coef,samples_shape_coef,pr_st0(k,:),10000);
        
        % 5% quantile of the predictive distribution
        qqTemp = quantile(samples_data,alpha);
        
        % compute expected shortfall
        samples_EsfST(ip,k) = mean(samples_data(samples_data<qqTemp));
        
        % Probability of negative growth
        samples_PrRec(ip,k) = mean(samples_data<0);
        
        % Quantile
        samples_quint(k,:,ip) = quantile(samples_data,p/100);
        
        %Tail Risk Scores
        samples_IC(ip,k)    = ytRealized(k)<qqTemp;
        samples_TLF(ip,k)   = (ytRealized(k)-qqTemp).*(alpha-(ytRealized(k)<=qqTemp));
        samples_GZF(ip,k)   = qqTemp*((ytRealized(k)<=qqTemp)-alpha)-ytRealized(k)*(ytRealized(k)<=qqTemp) ...
            + exp(samples_EsfST(ip,k)')/(1+exp(samples_EsfST(ip,k)'))*(samples_EsfST(ip,k)'-qqTemp+1/alpha*(qqTemp-ytRealized(k))*(ytRealized(k)<=qqTemp))+log(2/(1+exp(samples_EsfST(ip,k)')));
        
    end
        
end
fprintf('Elapsed time: %s.\n',secs2hms(toc))

% RMSE
samples_rmse = (ytRealized-mean(samples_mean,1)').^2;

% Save
save([save_root,'/models/',name_file,'/insample_stats.mat'], ...
    'samples_rmse','samples_score', ...
    'samples_mean','samples_variance','samples_skewness','samples_skewness_coef','samples_kurtosis','samples_kurtosis_coef', ...
    'samples_EsfST','samples_EljST','samples_PrRec', 'samples_quint', ...
    'samples_IC','samples_TLF','samples_GZF','samples_ALS', 'samples_pst', ...
    'Dat','yt');

