%
% Code to replicate Lhuissier (2022): "Financial Conditions and 
% Macroeconomic Downside Risks in the Euro Area", European Economic Review
%
% --> Maximization Procedure
%
%
% Stephane Lhuissier, August 2021

% -------------------------------------------------------------------------
% Housekeeping
% -------------------------------------------------------------------------

clear variables; clc;

addpath 'C:/dynare/4.2.2/matlab/'
addpath 'library/'
addpath 'prior/'

% -------------------------------------------------------------------------
% Options and Data
% -------------------------------------------------------------------------

% Model Specification
name_file = 'benchmark';
% Name of Data
data_file = 'ea_data_99q1_19q4.txt';
% Name of Endogenous Variable
yt_str    = {'RGDP'};
% Name of Lagged Variables
zt_str    = {'RGDP','CISS'};
% Beginning of Estimation
BinDat    = datetime('04/01/1999','InputFormat','MM/dd/yyyy');
% End of Estimation
EndDat    = datetime('10/01/2019','InputFormat','MM/dd/yyyy');

% -------------------------------------------------------------------------
% Prepare Data, Prior, and Model
% -------------------------------------------------------------------------

% Time Series of Estimation Date
Dat       = BinDat:calquarters(1):EndDat;

% Choice of Variables
% --
newData   = importdata(['data/',data_file]);
varnames  = newData.textdata(1,2:end);
[~,i_yt]  = ismember(yt_str,varnames);
[~,i_zt]  = ismember(zt_str,varnames);
yt        = newData.data(:,i_yt);
zt        = newData.data(:,i_zt)-mean(newData.data(:,i_zt),'omitnan');

% Time Series of Sample Date
qdates    = datetime(newData.textdata(2:end,1),'InputFormat','MM/dd/yyyy');

% Data
options_.data       = [yt(find(BinDat==qdates):find(EndDat==qdates),:),zt(find(BinDat==qdates)-1:find(EndDat==qdates)-1,:)];

% Name of Likelihood and Prior
options_.name_lik   = 'ms3p_tvtp_lik';
options_.name_prior = strcat(name_file,'_prior');

% Prior
% --
eval(options_.name_prior);
options_.pshape   = prior(:,1);
options_.sp       = prior(:,2);
options_.pmean    = prior(:,3);
options_.pstdd    = prior(:,4);
options_.trspec   = prior(:,5:8);
options_.mask     = prior(:,9);
options_.fix      = prior(:,10);
options_.maskinv  = 1-options_.mask;

options_.poscal  = find(options_.mask); 
options_.posest  = (1:length(options_.sp)); 
options_.posest  = find( ismember(options_.posest(:),options_.poscal(:)) == 0 ); 
x0               = options_.sp(options_.posest);

% -------------------------------------------------------------------------
% Run Estimation
% -------------------------------------------------------------------------

% Options for C.S optimizer
% --
options_.nparam  = size(options_.sp,1);
options_.post    = 1;
H0               = 1e-4*eye(options_.nparam);
H0               = H0(options_.posest,options_.posest);
crit             = 1.0e-6;
nit              = 100;
gradient_method  = 2;
gradient_epsilon = 1e-6;

tic
% Numerical optimization by Christopher Sims : BFGS optimizer
[fh,xh,gh,H,itct,fcount,retcodeh] = csminwel1('min_post',invtrans(x0,options_.trspec(options_.posest,:)),H0,[],crit,nit, gradient_method, gradient_epsilon, options_)
toc

xh = trans(xh,options_.trspec(options_.posest,:));

Postmode                  = zeros(options_.nparam,1);
Postmode(options_.posest) = xh;
Postmode                  = Postmode.*(1-options_.mask)+options_.fix.*options_.mask;

% Recover real-time probabilities and conditional likelihood
[lnlik, pr_st0, pr_stl0, Q, lnlik_store] =  feval(options_.name_lik, Postmode, options_.data);

% Compute prior
lnprior = priodens(xh, options_.pmean(options_.posest), ...
                       options_.pstdd(options_.posest), ...
                       options_.pshape(options_.posest));

% -------------------------------------------------------------------------
% Results
% -------------------------------------------------------------------------

% Save results
% --
filename=['results/models/',name_file,'/posterior_mode.mat'];
save(filename,'xh','pr_st0','options_','Dat','Postmode','Q');

% Display filtered probabilities 
% --
figure
subplot(3,1,1)
plot(Dat,pr_st0(:,1)+pr_st0(:,2)+pr_st0(:,5)+pr_st0(:,6))
xlim([Dat(1) Dat(end)])
ylim([-0.05 1.05])
xlabel('years')
ylabel('prob.')
box on
grid on
subplot(3,1,2)
plot(Dat,pr_st0(:,1)+pr_st0(:,3)+pr_st0(:,5)+pr_st0(:,7))
ylim([-0.05 1.05])
xlim([Dat(1) Dat(end)])
xlabel('years')
ylabel('prob.')
box on
grid on
subplot(3,1,3)
plot(Dat,pr_st0(:,1)+pr_st0(:,2)+pr_st0(:,3)+pr_st0(:,4))
xlim([Dat(1) Dat(end)])
ylim([-0.05 1.05])
xlabel('years')
ylabel('prob.')
box on
grid on

% Display coefficients
% --
disp(' ')
disp('----------------------------------------------------------------')
disp('Empirical moments')
disp('----------------------------------------------------------------')
cnames={'Mode'};
pnames={'mean(1)','mean(2)', ...
         'std(1)','std(2)', ...
         'skew(1)','skew(2)', ...
         'phi1', ...
         'a1(shape)','a2(shape)', ...
         'b1(shape,gdp)','b1(shape,ciss)', ...
         'b2(shape,gdp)','b2(shape,ciss)', ...
         'a1(location)','a2(location)', ...
         'b1(location,gdp)','b1(location,ciss)', ...
         'b2(location,gdp)','b2(location,ciss)', ...
         'a1(scale)','a2(scale)', ...
         'b1(scale,gdp)','b1(scale,ciss)', ...
         'b2(scale,gdp)','b2(scale,ciss)'};
r=printresults3(xh,cnames{1},pnames(options_.posest)); disp(r)
T=table(pnames(options_.posest)',xh,'VariableNames',{'Parameters','Mode'});
writetable(T,['results/models/',name_file,'/posterior_mode.xlsx']);

% Display moments
% --
location = [xh(1); xh(1); xh(2); xh(2); xh(1); xh(1); xh(2); xh(2)]'; 
scale    = 1./sqrt([xh(3); xh(4); xh(3); xh(4); xh(3); xh(4); xh(3); xh(4)])';
if ~strcmp(name_file,"normal")
    shape    = [xh(5); xh(5); xh(5); xh(5); xh(6); xh(6); xh(6); xh(6)]';
else
    shape    = zeros(1,size(pr_st0,2));
end

[expectation,variance,skewness,skewness_coeff,kurtosis,kurtosis_coeff] = ms_skn_moments(pr_st0, location, scale, shape);

figure;
subplot(2,3,1)
plot(Dat,expectation)
title('Mean')
subplot(2,3,2)
plot(Dat,variance)
title('Variance')
subplot(2,3,3)
plot(Dat,skewness)
title('Skewness')
subplot(2,3,4)
plot(Dat,skewness_coeff)
title('Skewness coefficient')
subplot(2,3,5)
plot(Dat,kurtosis)
title('Kurtosis')
subplot(2,3,6)
plot(Dat,kurtosis_coeff)
title('Kurtosis coefficient')
