%
% Code to replicate Lhuissier (2022): "Financial Conditions and
% Macroeconomic Downside Risks in the Euro Area", European Economic Review
%
% --> Out-of-sample Forecasting Procedure (iteration procedure)
%
%
% Stephane Lhuissier, August 2021

% -------------------------------------------------------------------------
% Housekeeping
% -------------------------------------------------------------------------

clear variables; clc;

addpath 'library/'
addpath 'library/skn-library/'

% -------------------------------------------------------------------------
% Settings
% -------------------------------------------------------------------------

% Model Specification
name_file = 'benchmark';

% Saved Gibbs results
results_root = 'results/';       % Root folder with MCMC results
save_root = 'results/';    % Root folder to save computations in

% Beginning of Real-Time
IntDat    = datetime('04/01/2010','InputFormat','MM/dd/yyyy');

% End of Estimation
EndDat    = datetime('10/01/2019','InputFormat','MM/dd/yyyy');

% Time Series of Estimation Date
DatRT     = IntDat:calquarters(1):EndDat;

% Number of estimation sequence
n         = length(DatRT);

% Expected shortfall
lower_quant = 0.05;              % Quantile used for computing expected shortfall

% h-step-ahead forecast
h = 1;                           % Forecast horizon (set to [] if h-step-ahead forecast not desired)
numdraw_h = 5e3;                 % Number of posterior parameter draws to consider
numsim_h = 1e3;                  % Number of forecast simulations per posterior parameter draw
rng(123);                        % Random number seed for forecast simulations

deltaYY = 2^9;                   % the number of mesh points 
Y_min   = -15;
Y_max   = 15;
YY      = linspace(Y_min,Y_max,deltaYY);

p = [0.05 0.16 0.50 0.84 0.95];

% Number of workers
nworkers = 16;

% -------------------------------------------------------------------------
% Load MCMC results
% -------------------------------------------------------------------------

fprintf('%s%d%s\n', 'h = ', h, ': Forecasting procedure starts here.');

tic
for iter=1:n-h
    % Model structure
    M = getModel(DatRT(iter),name_file);
    
    %
    ytRealized = M.ytTab.Var1(find(M.ytTab.Time==DatRT(iter))+1,1);
    
    % Gibbs output
    load([results_root,'models/',name_file,'/real-time/gibbs/',datestr(DatRT(iter)),'.mat'], ...
        'a1gbs','a2gbs','b11gbs','b12gbs','b21gbs','b22gbs',...
        'locationgbs','phigbs','scalegbs','shapegbs','sgbs');
    
    zt = M.ztTab.Var1(find(M.ztTab.Time==DatRT(iter)),:);
    %Dat = M.Dat;
    %T = size(M.yt,1);
    numdraw = size(locationgbs,1);
    
    % -------------------------------------------------------------------------
    % h steps ahead: Simulations
    % -------------------------------------------------------------------------
    
    samples_mean_h           = nan(numdraw_h,1);
    samples_std_h            = nan(numdraw_h,1);
    samples_skew_h           = nan(numdraw_h,1);
    samples_kurt_h           = nan(numdraw_h,1);
    samples_prob_neg         = nan(numdraw_h,1);
    samples_prob_belowmean   = nan(numdraw_h,1);
    samples_shortfall        = nan(numdraw_h,1);
    samples_score            = nan(numdraw_h,1);
    samples_Pit              = nan(numdraw_h,1);
    samples_TLF              = nan(numdraw_h,1);
    samples_GZF              = nan(numdraw_h,1);
    samples_ALS              = nan(numdraw_h,1);
    samples_IC               = nan(numdraw_h,1);
    samples_pst              = nan(numdraw_h,length(YY));
    samples_quint            = nan(numdraw_h,length(p));
    
    if ~isempty(h)
        %for ip=1:numdraw_h
        parfor(ip=1:numdraw_h,nworkers)
           
            % load parameters
            samples_location_coef_h = locationgbs(ip,:);
            samples_scale_coef_h    = scalegbs(ip,:);
            samples_shape_coef_h    = shapegbs(ip,:);
            samples_s_coef_h        = sgbs(ip,:);
            samples_a1_coef_h       = a1gbs(ip,:);
            samples_a2_coef_h       = a2gbs(ip,:);
            samples_b11_coef_h      = b11gbs(ip,:);
            samples_b12_coef_h      = b12gbs(ip,:);
            samples_b21_coef_h      = b21gbs(ip,:);
            samples_b22_coef_h      = b22gbs(ip,:);
            
            % Evaluate transition matrix
            P = tran_p2q(samples_a1_coef_h,samples_a2_coef_h,[samples_b11_coef_h;samples_b12_coef_h],[samples_b21_coef_h;samples_b22_coef_h],zt);
            P_cum = cumsum(P,1);
            ns    = size(P,1);
            
            % Simulate h steps
            sim_y_cum = zeros(numsim_h,1);
            for is=1:numsim_h
                
                xit_1 = samples_s_coef_h(end); %Starting from the current regime.
                
                the_y_cum = 0;
                for l=1:h % Simulate forward one step at a time
                    
                    %--- Generating exogenous regimes.
                    sdraw = rand;
                    for si=1:ns
                        if (sdraw<P_cum(si,xit_1))
                            xit = si;
                            break;
                        end
                    end
                   
                    %--- Generating GDP growth.
                    the_y = rsn(1,samples_location_coef_h(xit),samples_scale_coef_h(xit),samples_shape_coef_h(xit));
                    the_y_cum = the_y_cum + the_y;
                    
                    %--- Reset for the next period.
                    xit_1 = xit;
                   
                end
                sim_y_cum(is) = the_y_cum; % Store simulated y_{t+1}+...+y_{t+h}
                
            end
            
            % Kernel-smoothed estimate of the draws
            [~,densityhat,xmesh,cdfhat]=kde(sim_y_cum,deltaYY,Y_min,Y_max);
            
            % Find index of GDP outcome
            [~,ix] = min(abs(xmesh-ytRealized));
            
            % Log predictive score
            samples_score(ip) = densityhat(ix);
            
            % Out-of-sample predictive densities
            samples_pst(ip,:) = densityhat;
            
            % Compute moments of y_{t+1}+...+y_{t+h}
            samples_mean_h(ip) = mean(sim_y_cum);
            samples_std_h(ip)  = std(sim_y_cum);
            samples_skew_h(ip) = skewness(sim_y_cum);
            samples_kurt_h(ip) = kurtosis(sim_y_cum);
            
            % 5% quantile of the predictive distribution
            qqTemp = quantile(sim_y_cum,lower_quant);
          
            % Compute other statistics
            samples_prob_neg(ip)   = mean(sim_y_cum<0);
            samples_shortfall(ip)  = mean(sim_y_cum(sim_y_cum<qqTemp));
            samples_quint(ip,:)    = quantile(sim_y_cum,p);
            samples_Pit(ip)        = cdfhat(ix);
        
            %Tail Risk Scores
            samples_IC(ip)    = ytRealized<qqTemp;
            samples_TLF(ip)   = (ytRealized-qqTemp)*(lower_quant-(ytRealized<=qqTemp));
            samples_GZF(ip)   = qqTemp*((ytRealized<=qqTemp)-lower_quant)-ytRealized*(ytRealized<=qqTemp) ...
                                + exp(samples_shortfall(ip))/(1+exp(samples_shortfall(ip)))*(samples_shortfall(ip)-qqTemp+1/lower_quant*(qqTemp-ytRealized)*(ytRealized<=qqTemp))+log(2/(1+exp(samples_shortfall(ip))));   
        end
        
        % RMSE
        samples_rmse = (ytRealized-mean(samples_mean_h))^2;
    end
    
    % Save
    save([save_root,'/models/',name_file,'/real-time/outofsample/',datestr(DatRT(iter)),'.mat'], ...
        'samples_mean_h','samples_std_h','samples_skew_h','samples_kurt_h', ...
        'samples_prob_neg','samples_shortfall','h','lower_quant', ...
        'samples_TLF','samples_GZF','samples_ALS','samples_IC','ytRealized', ...
        'YY','samples_pst','samples_score','samples_Pit','samples_rmse','samples_quint');
end
fprintf('\nElapsed time: %s.\n',secs2hms(toc))

